@@ -88,6 +88,13 @@ fn cli<'a>() -> App<'a> {
                 .default_value(DEFAULT_PRINTER)
                 .about("Printer to print the match results. 'bat' or 'syntect' is available"),
         )
+        .arg(
+            Arg::new("term-width")
+                .long("term-width")
+                .takes_value(true)
+                .value_name("NUM")
+                .about("Width (number of characters) of terminal window"),
+        )
         .arg(
             Arg::new("generate-completion-script")
                 .long("generate-completion-script")
@@ -96,6 +103,18 @@ fn cli<'a>() -> App<'a> {
                 .about("Print completion script for SHELL to stdout. SHELL must be one of 'bash', 'zsh', 'powershell', 'fish', or 'elvish'"),
         );
 
+    #[cfg(feature = "bat-printer")]
+    let app = app
+        .arg(
+            Arg::new("custom-assets")
+                .long("custom-assets")
+                .about("Load bat's custom assets. Note that this flag may not work with some version of `bat` command. This flag is only for bat printer"),
+        ). arg(
+            Arg::new("no-wrap")
+                .long("no-wrap")
+                .about("Disable the text-wrapping mode. This flag is only for bat printer")
+        );
+
     #[cfg(feature = "syntect-printer")]
     let app = app.arg(
         Arg::new("background")
@@ -363,6 +382,16 @@ fn app() -> Result<bool> {
         printer_opts.grid = false;
     }
 
+    if let Some(width) = matches.value_of("term-width") {
+        let width = width
+            .parse()
+            .context("could not parse \"term-width\" option value as unsigned integer")?;
+        printer_opts.term_width = width;
+        if width < 10 {
+            anyhow::bail!("Too small value at --term-width option ({} < 10)", width);
+        }
+    }
+
     #[cfg(feature = "syntect-printer")]
     if matches.is_present("background") {
         printer_opts.background_color = true;
@@ -372,6 +401,24 @@ fn app() -> Result<bool> {
         }
     }
 
+    #[cfg(feature = "bat-printer")]
+    if matches.is_present("custom-assets") {
+        printer_opts.custom_assets = true;
+        #[cfg(feature = "syntect-printer")]
+        if printer_kind == PrinterKind::Syntect {
+            anyhow::bail!("--custom-assets flag is only available for bat printer");
+        }
+    }
+
+    #[cfg(feature = "bat-printer")]
+    if matches.is_present("no-wrap") {
+        printer_opts.text_wrap = false;
+        #[cfg(feature = "syntect-printer")]
+        if printer_kind == PrinterKind::Syntect {
+            anyhow::bail!("--no-wrap option is only available for bat printer");
+        }
+    }
+
     #[cfg(feature = "ripgrep")]
     if let Some(pattern) = matches.value_of("PATTERN") {
         let paths = matches.values_of_os("PATH");
