@@ -1,13 +1,11 @@
 use crate::chunk::File;
 use crate::chunk::Line;
-use crate::printer::{Printer, PrinterOptions};
+use crate::printer::{Printer, PrinterOptions, TermColorSupport};
 use anyhow::Result;
-use console::Term;
 use memchr::{memchr_iter, Memchr};
 use rgb2ansi256::rgb_to_ansi256;
 use std::cmp;
 use std::collections::HashSet;
-use std::env;
 use std::ffi::OsStr;
 use std::fmt;
 use std::io::{self, Stdout, StdoutLock};
@@ -17,7 +15,6 @@ use syntect::highlighting::{
     Color, FontStyle, HighlightIterator, HighlightState, Highlighter, Style, Theme, ThemeSet,
 };
 use syntect::parsing::{ParseState, ScopeStack, SyntaxReference, SyntaxSet};
-use term::terminfo::TermInfo;
 use unicode_width::UnicodeWidthStr;
 
 // Note for lifetimes:
@@ -54,36 +51,6 @@ pub fn list_themes<W: Write>(mut out: W) -> Result<()> {
     Ok(())
 }
 
-#[derive(Clone, Copy, PartialEq, Eq)]
-enum TermColorSupport {
-    True,
-    Ansi256,
-    Ansi16,
-}
-
-impl TermColorSupport {
-    fn detect() -> Self {
-        if env::var("COLORTERM")
-            .ok()
-            .map(|v| v.eq_ignore_ascii_case("truecolor"))
-            .unwrap_or(false)
-        {
-            return TermColorSupport::True;
-        }
-
-        if let Ok(info) = TermInfo::from_env() {
-            if let Some(colors) = info.numbers.get("colors") {
-                if *colors < 256 {
-                    return TermColorSupport::Ansi16;
-                }
-            }
-        }
-
-        // Assume 256 colors by default (I'm not sure this is correct)
-        TermColorSupport::Ansi256
-    }
-}
-
 // Use u64::log10 once it is stabilized: https://github.com/rust-lang/rust/issues/70887
 #[inline]
 fn num_digits(n: u64) -> u16 {
@@ -357,7 +324,9 @@ impl<'file, W: Write> Drawer<'file, W> {
         for _ in 0..self.term_width - gutter_width + 1 {
             self.out.write_all("─".as_bytes())?;
         }
-        self.reset_color()
+        self.reset_color()?;
+        writeln!(self.out)?;
+        Ok(())
     }
 
     fn reset_color(&mut self) -> Result<()> {
@@ -566,8 +535,8 @@ impl<'file, W: Write> Drawer<'file, W> {
             self.fill_rest_with_spaces(path.width_cjk() + 1)?;
         } else {
             self.reset_color()?;
-            writeln!(self.out)?;
         }
+        writeln!(self.out)?;
         if self.grid {
             self.draw_horizontal_line("┬")?;
         }
@@ -580,13 +549,9 @@ impl<'file, W: Write> Drawer<'file, W> {
         }
         Ok(())
     }
-}
 
-impl<'file, W: Write> Drop for Drawer<'file, W> {
-    fn drop(&mut self) {
-        self.out
-            .flush()
-            .expect("could not flush stdout for syntect printer");
+    fn flush(&mut self) -> Result<()> {
+        Ok(self.out.flush()?)
     }
 }
 
@@ -612,12 +577,10 @@ pub struct SyntectPrinter<'main, W>
 where
     for<'a> W: LockableWrite<'a>,
 {
-    stdout: W, // Protected with mutex because it should print file by file
+    writer: W, // Protected with mutex because it should print file by file
     syntaxes: SyntaxSet,
     themes: ThemeSet,
     opts: PrinterOptions<'main>,
-    term_width: u16,
-    term_support: TermColorSupport,
 }
 
 impl<'main> SyntectPrinter<'main, Stdout> {
@@ -632,15 +595,17 @@ where
 {
     pub fn new(out: W, opts: PrinterOptions<'main>) -> Result<Self> {
         Ok(Self {
-            stdout: out,
+            writer: out,
             syntaxes: bincode::deserialize_from(flate2::read::ZlibDecoder::new(SYNTAX_SET_BIN))?,
             themes: load_themes(opts.theme)?,
             opts,
-            term_width: Term::stdout().size().1,
-            term_support: TermColorSupport::detect(),
         })
     }
 
+    pub fn writer_mut(&mut self) -> &mut W {
+        &mut self.writer
+    }
+
     fn parse_highlights<'file>(
         &self,
         file: &'file File,
@@ -720,20 +685,20 @@ where
             lines,
             theme,
             grid: self.opts.grid,
-            term_width: self.term_width,
+            term_width: self.opts.term_width,
             lnum_width,
             tab_width: self.opts.tab_width as u16,
             background: self.opts.background_color,
             gutter_color,
             match_color: theme.settings.line_highlight.or(theme.settings.background),
-            true_color: self.term_support == TermColorSupport::True,
-            out: BufWriter::new(self.stdout.lock()), // Take lock here to print files in serial from multiple threads
+            true_color: self.opts.color_support == TermColorSupport::True,
+            out: BufWriter::new(self.writer.lock()), // Take lock here to print files in serial from multiple threads
         }
     }
 
     fn theme(&self) -> &Theme {
         let name = self.opts.theme.unwrap_or_else(|| {
-            if self.term_support == TermColorSupport::Ansi16 {
+            if self.opts.color_support == TermColorSupport::Ansi16 {
                 "ansi"
             } else {
                 "Monokai Extended" // Our 25bit -> 8bit color conversion works really well with this colorscheme
@@ -776,6 +741,214 @@ where
         let mut drawer = self.build_drawer(highlighted, theme, include_separator); // Lock is acquired here
         drawer.draw_header(&file.path)?;
         drawer.draw_lines()?;
-        drawer.draw_footer()
+        drawer.draw_footer()?;
+        drawer.flush()
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::chunk::File;
+    use std::cell::{RefCell, RefMut};
+    use std::fmt;
+    use std::fs;
+    use std::mem;
+    use std::path::PathBuf;
+
+    struct DummyStdoutLock<'a>(RefMut<'a, Vec<u8>>);
+    impl<'a> Write for DummyStdoutLock<'a> {
+        fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
+            self.0.write(buf)
+        }
+        fn flush(&mut self) -> io::Result<()> {
+            self.0.flush()
+        }
+    }
+
+    #[derive(Default)]
+    struct DummyStdout(RefCell<Vec<u8>>);
+    impl<'a> LockableWrite<'a> for DummyStdout {
+        type Locked = DummyStdoutLock<'a>;
+        fn lock(&'a self) -> Self::Locked {
+            DummyStdoutLock(self.0.borrow_mut())
+        }
+    }
+
+    #[cfg(not(windows))]
+    mod uitests {
+        use super::*;
+        use std::path::Path;
+
+        fn read_chunks(path: PathBuf) -> File {
+            let contents = fs::read(&path).unwrap();
+            let mut lnums = vec![];
+            let mut chunks = vec![];
+            for (idx, line) in contents.split_inclusive(|b| *b == b'\n').enumerate() {
+                let lnum = (idx + 1) as u64;
+                let pat = "*match to this line*".as_bytes();
+                if line.windows(pat.len()).any(|s| s == pat) {
+                    lnums.push(lnum);
+                    chunks.push((lnum.saturating_sub(6), lnum + 6));
+                }
+            }
+            File::new(path, lnums, chunks, contents)
+        }
+
+        fn run_uitest(infile: PathBuf, outfile: PathBuf, f: fn(&mut PrinterOptions<'_>) -> ()) {
+            let stdout = DummyStdout(RefCell::new(vec![]));
+            let mut opts = PrinterOptions::default();
+            opts.term_width = 80;
+            opts.color_support = TermColorSupport::True;
+            f(&mut opts);
+            let mut printer = SyntectPrinter::new(stdout, opts).unwrap();
+            let file = read_chunks(infile);
+            printer.print(file).unwrap();
+            let printed = mem::take(printer.writer_mut()).0.into_inner();
+            let expected = fs::read(outfile).unwrap();
+            assert_eq!(
+                printed,
+                expected,
+                "got:\n{}\nwant:\n{}",
+                String::from_utf8_lossy(&printed),
+                String::from_utf8_lossy(&expected),
+            );
+        }
+
+        fn run_parametrized_uitest(mut input: &str, f: fn(&mut PrinterOptions<'_>) -> ()) {
+            let dir = Path::new(".").join("testdata").join("syntect");
+            if input.starts_with("test_") {
+                input = &input["test_".len()..];
+            }
+            let infile = dir.join(format!("{}.rs", input));
+            let outfile = dir.join(format!("{}.out", input));
+            run_uitest(infile, outfile, f);
+        }
+
+        macro_rules! uitest {
+            ($($input:ident($f:expr),)+) => {
+                $(
+                    #[cfg(not(windows))]
+                    #[test]
+                    fn $input() {
+                        run_parametrized_uitest(stringify!($input), $f);
+                    }
+                )+
+            }
+        }
+
+        uitest!(
+            test_default(|_| {}),
+            test_background(|o| {
+                o.background_color = true;
+            }),
+            test_no_grid(|o| {
+                o.grid = false;
+            }),
+            test_theme(|o| {
+                o.theme = Some("Nord");
+            }),
+            test_tab_width_2(|o| {
+                o.tab_width = 2;
+            }),
+            test_hard_tab(|o| {
+                o.tab_width = 0;
+            }),
+            test_ansi256_colors(|o| {
+                o.color_support = TermColorSupport::Ansi256;
+            }),
+            test_ansi16_colors(|o| {
+                o.color_support = TermColorSupport::Ansi16;
+            }),
+            test_long_line(|_| {}),
+            test_long_line_bg(|o| {
+                o.background_color = true;
+            }),
+        );
+    }
+
+    #[derive(Debug)]
+    struct DummyError;
+    impl std::error::Error for DummyError {}
+    impl fmt::Display for DummyError {
+        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+            write!(f, "dummy error!")
+        }
+    }
+
+    struct ErrorStdoutLock;
+    impl Write for ErrorStdoutLock {
+        fn write(&mut self, _: &[u8]) -> io::Result<usize> {
+            Err(io::Error::new(io::ErrorKind::Other, DummyError))
+        }
+        fn flush(&mut self) -> io::Result<()> {
+            Ok(())
+        }
+    }
+
+    #[derive(Default)]
+    struct ErrorStdout;
+    impl<'a> LockableWrite<'a> for ErrorStdout {
+        type Locked = ErrorStdoutLock;
+        fn lock(&'a self) -> Self::Locked {
+            ErrorStdoutLock
+        }
+    }
+
+    fn readme_chunk() -> File {
+        let readme = PathBuf::from("README.md");
+        let lnums = vec![3];
+        let chunks = vec![(1, 6)];
+        let contents = fs::read(&readme).unwrap();
+        File::new(readme, lnums, chunks, contents)
+    }
+
+    #[test]
+    fn test_error_write() {
+        let file = readme_chunk();
+        let opts = PrinterOptions::default();
+        let printer = SyntectPrinter::new(ErrorStdout, opts).unwrap();
+        let err = printer.print(file).unwrap_err();
+        assert_eq!(&format!("{}", err), "dummy error!", "message={}", err);
+    }
+
+    #[test]
+    fn test_unknown_theme() {
+        let mut opts = PrinterOptions::default();
+        opts.theme = Some("this theme does not exist");
+        let err = match SyntectPrinter::with_stdout(opts) {
+            Err(e) => e,
+            Ok(_) => panic!("error did not occur"),
+        };
+        let msg = format!("{}", err);
+        assert!(msg.contains("Unknown theme"), "message={:?}", msg);
+    }
+
+    #[test]
+    fn test_list_themes() {
+        let mut buf = vec![];
+        list_themes(&mut buf).unwrap();
+        let out = String::from_utf8(buf).unwrap();
+
+        // From bat's assets
+        assert!(out.contains("Monokai Extended\n"), "output={:?}", out);
+
+        // From default assets
+        assert!(out.contains("base16-ocean.dark\n"), "output={:?}", out);
+    }
+
+    #[test]
+    fn test_print_nothing() {
+        let file = File::new(PathBuf::from("x.txt"), vec![], vec![], vec![]);
+        let opts = PrinterOptions::default();
+        let stdout = DummyStdout(RefCell::new(vec![]));
+        let mut printer = SyntectPrinter::new(stdout, opts).unwrap();
+        printer.print(file).unwrap();
+        let printed = mem::take(printer.writer_mut()).0.into_inner();
+        assert!(
+            printed.is_empty(),
+            "pritned:\n{}",
+            String::from_utf8_lossy(&printed)
+        );
     }
 }
